----------------------------------------------------------------------------------
--
-- Local clocks
--
-- 100 MHz input clock (Numato Mimas system clock);
--  60 MHz USB PHY layer clock
--  10 MHz system/IO clock output
-- 125 kHz (variable rate) LGP-30 simulation clock output 
-- 
-- jm 1.8.2016
--
----------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use IEEE.math_real.all;

library work;
use work.constants.all;

library unisim;
use unisim.vcomponents.all;

entity clock is
	Port ( 
		clk_ext: 	in  std_logic;				-- external clock input
		clk_usb:		out std_logic;				-- USB clock, 60 MHz
		clk_io: 		buffer std_logic;			-- I/O clock, 10 MHz
		clk_lgp30:	out std_logic;				-- LGP-30 CPU clock. High lasts one I/O clock cycle
		clk_lgp30_unbuf: out std_logic;		-- same, but unbuffered - for use in io clock and lgp clk synchronization in panel and typewriter modules
			
		-- control for LGP-30 clock rate
		cup, cdn:	in std_logic;				-- pulse high for one clk_io cycle
		cstep:		in std_logic;				-- set high to force clock speed to single step (XModem mode)
		cpow: 		out freq_index				-- slowing clock down by 2^cdiv
	);
end clock;

architecture xilinx of clock is
	attribute CORE_GENERATION_INFO : string;
	attribute CORE_GENERATION_INFO of xilinx : architecture is "clocks,clk_wiz_v3_6,{component_name=clocks,use_phase_alignment=true,use_min_o_jitter=false,use_max_i_jitter=false,use_dyn_phase_shift=false,use_inclk_switchover=false,use_dyn_reconfig=false,feedback_source=FDBK_AUTO,primtype_sel=DCM_SP,num_out_clk=1,clkin1_period=10.0,clkin2_period=10.0,use_power_down=false,use_reset=true,use_locked=true,use_inclk_stopped=false,use_status=false,use_freeze=false,use_clk_valid=false,feedback_type=SINGLE,clock_mgr_type=AUTO,manual_override=false}";

	-- Input clock buffering / unused connectors
	signal clkin1            : std_logic;
	
	-- Output clock buffering
	signal clkfb             : std_logic;
	signal clk0              : std_logic;
	signal clkfx             : std_logic;
	signal clkfbout          : std_logic;
	signal locked_internal   : std_logic;
	signal status_internal   : std_logic_vector(7 downto 0);

	-- derive 125 kHz clock 

	signal clk_slow: std_logic;										-- unbuffered 125 kHz clock signal
			
	signal clk_divider: natural range 0 to clk_div_max := clk_div_std;	-- current init value for clk_count
	signal clk_power: natural range cpow_min to cpow_max := cpow_std;		-- current clock speed index
	signal clk_count: natural range 0 to clk_div_max;				-- clock divide counter

begin

  -- Input buffering
  --------------------------------------
  -- clkin1_buf : IBUFG port map (O => clkin1, I => CLK_EXT);	-- disabled, clock is buffered globally

	clkin1 <= clk_ext;

  -- Clocking primitive
  --------------------------------------
  
  -- Instantiation of the DCM primitive
  --    * Unused inputs are tied off
  --    * Unused outputs are labeled unused
  
  dcm_sp_inst: DCM_SP
  generic map
   (CLKDV_DIVIDE          => real(CLK_EXT_RATE)/real(CLK_IO_RATE),
    CLKFX_DIVIDE          => 2 * CLK_EXT_RATE/CLK_IO_RATE,
    CLKFX_MULTIPLY        => 2,
    CLKIN_DIVIDE_BY_2     => FALSE,
    CLKIN_PERIOD          => real(1E3)/real(CLK_EXT_RATE),
    CLKOUT_PHASE_SHIFT    => "NONE",
    CLK_FEEDBACK          => "1X",
    DESKEW_ADJUST         => "SYSTEM_SYNCHRONOUS",
    PHASE_SHIFT           => 0,
    STARTUP_WAIT          => FALSE)
  port map
   -- Input clock
   (CLKIN                 => clkin1,
    CLKFB                 => clkfb,
    -- Output clocks
    CLK0                  => clk0,
    CLK90                 => open,
    CLK180                => open,
    CLK270                => open,
    CLK2X                 => open,
    CLK2X180              => open,
    CLKFX                 => clkfx,
    CLKFX180              => open,
    CLKDV                 => open,
   -- Ports for dynamic phase shift
    PSCLK                 => '0',
    PSEN                  => '0',
    PSINCDEC              => '0',
    PSDONE                => open,
   -- Other control and status signals
    LOCKED                => locked_internal,
    STATUS                => status_internal,
    RST                   => '0',
   -- Unused pin, tie low
    DSSEN                 => '0');


	-- LGP-30 clock: divide 10 MHz IO clock.
	-- Note: Asymmetrical clock; high only for one IO clock cycle!
	-- This is used by the peripheral modules to run timers at CPU clock rate.
	
	process (clk_io) is
	begin
		if rising_edge (clk_io) then
		
			-- create LGP-30 clock
			if (clk_power = cpow_step) or (cstep = '1') then	-- single step mode
				clk_slow <= cdn;								-- clock ticks when encoder pulses
			else													-- regular clock
				if clk_count = 0 then                  -- counts down to divide clock
				  clk_count <= clk_divider -1;
				  clk_slow  <= '1';                    -- single cycle enable pulse
				 else
				  clk_count <= clk_count-1;
				  clk_slow <= '0';
				end if;
			end if;

			-- does the user want to change the clock rate?
			if (cup='1') and (clk_power < cpow_max) and (cstep = '0') then
				clk_divider <= freq_divider (clk_power+1);
				clk_power <= clk_power+1;
			elsif (cdn='1') and (clk_power > cpow_step)  and (cstep = '0') then
				clk_divider <= freq_divider (clk_power-1);
				clk_power <= clk_power-1;
			end if;

			-- indicate current (most recent, to be precise) clock rate
			if (cstep='0') then cpow <= clk_power;
			else cpow <= cpow_min;
			end if;
			
		end if;
	end process;
	
  -- Output buffering
  -------------------------------------
  clkf_buf : BUFG
  port map
   (O => clkfb,
    I => clk0);

  clkout1_buf : BUFG
  port map
   (O   => CLK_IO,
    I   => clkfx);

  clkout2_buf : BUFG
  port map
   (O   => CLK_lgp30,
    I   => clk_slow);
	 
  clk_lgp30_unbuf <= clk_slow;

end xilinx;
