----------------------------------------------------------------------------------

-- LGP-30 drum computer simulation
-- Global constants
--
-- jm 7.9.2016
-- jm 30.7.2017 USB clock rate added

----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;

package constants is

	constant CLK_EXT_RATE: natural := 100;
	constant CLK_IO_RATE: natural := 10;
	constant CLK_USB_RATE: natural := 60;

	-- variable rates for LGP-30 clock

	constant cpow_min: integer := 0;							-- min index (for slowest LGP-30 clock)
	constant cpow_max: integer := 20;						-- max index (for fastest LGP-30 clock)
	constant cpow_step: integer := 1;						-- index for single step mode
	constant cpow_std: integer := 17;						-- standard clock rate (as in real LGP-30)
	constant cpow_slow: integer := 9;						-- use animations at this clock speed and below
	
	constant clk_div_std: natural := (CLK_IO_RATE*1000)/125;			-- standard divider for 125 kHz clock
	constant clk_div_max: natural := (CLK_IO_RATE*100000000)/125;	-- maximum divider for slowest clock

	-- frequency display texts

	subtype freq_index is integer range cpow_min to cpow_max;
	type freq_text is array (0 to 5) of character;
	type freq_map is array (freq_index) of freq_text;
	type freq_div is array (freq_index) of natural;

	constant freq_list: freq_map :=
	(
		('X', 'M', 'o', 'd', 'e', 'm'),
		(' ', 'S', 't', 'e', 'p', ' '),
		('1', '.', '2', ' ', 'H', 'z'),
		('2', '.', '5', ' ', 'H', 'z'),
		(' ', ' ', '5', ' ', 'H', 'z'),
		(' ', '1', '2', ' ', 'H', 'z'),
		(' ', '2', '5', ' ', 'H', 'z'),
		(' ', '5', '0', ' ', 'H', 'z'),
		('1', '2', '5', ' ', 'H', 'z'),
		('2', '5', '0', ' ', 'H', 'z'),
		('5', '0', '0', ' ', 'H', 'z'),
		('1', '.', '2', 'k', 'H', 'z'),
		('2', '.', '5', 'k', 'H', 'z'),
		(' ', ' ', '5', 'k', 'H', 'z'),
		(' ', '1', '2', 'k', 'H', 'z'),
		(' ', '2', '5', 'k', 'H', 'z'),
		(' ', '5', '0', 'k', 'H', 'z'),
		('1', '2', '5', 'k', 'H', 'z'),
		('2', '5', '0', 'k', 'H', 'z'),
		('5', '0', '0', 'k', 'H', 'z'),
		('1', '.', '2', 'M', 'H', 'z')
	);

	constant freq_divider: freq_div :=
	(
		1,										-- XModem - divider not used
		1,										-- single step - divider not used
		(CLK_IO_RATE*100000000)/125,	-- 1.25 Hz
		(CLK_IO_RATE*100000000)/250,
		(CLK_IO_RATE*100000000)/500,
		(CLK_IO_RATE*10000000)/125,	-- 12.5 Hz
		(CLK_IO_RATE*10000000)/250,
		(CLK_IO_RATE*10000000)/500,
		(CLK_IO_RATE*1000000)/125,		-- 125 Hz
		(CLK_IO_RATE*1000000)/250,
		(CLK_IO_RATE*1000000)/500,
		(CLK_IO_RATE*100000)/125,		-- 1250 Hz
		(CLK_IO_RATE*100000)/250,
		(CLK_IO_RATE*100000)/500,
		(CLK_IO_RATE*10000)/125,		-- 12.5 kHz
		(CLK_IO_RATE*10000)/250,
		(CLK_IO_RATE*10000)/500,
		(CLK_IO_RATE*1000)/125,			-- 125 kHz
		(CLK_IO_RATE*1000)/250,
		(CLK_IO_RATE*1000)/500,
		(CLK_IO_RATE*1000)/1250			-- 1.25 MHz
	);

	function is_std_clock  (signal cpow : in freq_index) return std_logic;
	function is_slow_clock (signal cpow : in freq_index) return std_logic;

	-- ASCI character constants 
	-- Control activation of the XModem protocol (Flexowriter and XModem units)
	constant xmodem_start_tx: std_logic_vector (7 downto 0) := "00010101";	-- NAK: PC wants to receive
	constant xmodem_start_rx: std_logic_vector (7 downto 0) := "01011100";	-- '\': PC wants to send

	-- character generator
	-- Font by Benedikt K. http://www.mikrocontroller.net/topic/54860

	constant char_zero: natural := 6*48;	-- index to '0' character in char_gen table
	
	type char_map is array (0 to 6*128-1) of std_logic_vector (7 downto 0);
	constant char_gen: char_map := 
	(
		x"00", x"00", x"00", x"00", x"00", x"00", 	-- $00
		x"00", x"3E", x"45", x"51", x"45", x"3E", 	-- $01
		x"00", x"3E", x"6B", x"6F", x"6B", x"3E", 	-- $02
		x"00", x"1C", x"3E", x"7C", x"3E", x"1C", 	-- $03
		x"00", x"18", x"3C", x"7E", x"3C", x"18", 	-- $04
		x"00", x"30", x"36", x"7F", x"36", x"30", 	-- $05
		x"00", x"18", x"5C", x"7E", x"5C", x"18", 	-- $06
		x"00", x"00", x"00", x"00", x"00", x"00", 	-- $07
		x"00", x"00", x"00", x"00", x"00", x"00", 	-- $08
		x"00", x"00", x"00", x"00", x"00", x"00", 	-- $09
		x"00", x"00", x"00", x"00", x"00", x"00", 	-- $0A
		x"00", x"30", x"48", x"4A", x"36", x"0E", 	-- $0B
		x"00", x"06", x"29", x"79", x"29", x"06", 	-- $0C
		x"00", x"00", x"00", x"00", x"00", x"00", 	-- $0D
		x"00", x"60", x"7E", x"0A", x"35", x"3F", 	-- $0E
		x"00", x"2A", x"1C", x"36", x"1C", x"2A", 	-- $0F
		x"00", x"00", x"7F", x"3E", x"1C", x"08", 	-- $10
		x"00", x"08", x"1C", x"3E", x"7F", x"00", 	-- $11
		x"00", x"14", x"36", x"7F", x"36", x"14", 	-- $12
		x"00", x"00", x"5F", x"00", x"5F", x"00", 	-- $13
		x"00", x"06", x"09", x"7F", x"01", x"7F", 	-- $14
		x"00", x"22", x"4D", x"55", x"59", x"22", 	-- $15
		x"00", x"60", x"60", x"60", x"60", x"00", 	-- $16
		x"00", x"14", x"B6", x"FF", x"B6", x"14", 	-- $17
		x"00", x"04", x"06", x"7F", x"06", x"04", 	-- $18
		x"00", x"10", x"30", x"7F", x"30", x"10", 	-- $19
		x"00", x"08", x"08", x"3E", x"1C", x"08", 	-- $1A
		x"00", x"08", x"1C", x"3E", x"08", x"08", 	-- $1B
		x"00", x"78", x"40", x"40", x"40", x"40", 	-- $1C
		x"00", x"08", x"3E", x"08", x"3E", x"08", 	-- $1D
		x"00", x"30", x"3C", x"3F", x"3C", x"30", 	-- $1E
		x"00", x"03", x"0F", x"3F", x"0F", x"03", 	-- $1F
		x"00", x"00", x"00", x"00", x"00", x"00", 	-- $20
		x"00", x"00", x"06", x"5F", x"06", x"00", 	-- $21
		x"00", x"07", x"03", x"00", x"07", x"03", 	-- $22
		x"00", x"24", x"7E", x"24", x"7E", x"24", 	-- $23
		x"00", x"24", x"2B", x"6A", x"12", x"00", 	-- $24
		x"00", x"63", x"13", x"08", x"64", x"63", 	-- $25
		x"00", x"36", x"49", x"56", x"20", x"50", 	-- $26
		x"00", x"00", x"07", x"03", x"00", x"00", 	-- $27
		x"00", x"00", x"3E", x"41", x"00", x"00", 	-- $28
		x"00", x"00", x"41", x"3E", x"00", x"00", 	-- $29
		x"00", x"08", x"3E", x"1C", x"3E", x"08", 	-- $2A
		x"00", x"08", x"08", x"3E", x"08", x"08", 	-- $2B
		x"00", x"00", x"E0", x"60", x"00", x"00", 	-- $2C
		x"00", x"08", x"08", x"08", x"08", x"08", 	-- $2D
		x"00", x"00", x"60", x"60", x"00", x"00", 	-- $2E
		x"00", x"20", x"10", x"08", x"04", x"02", 	-- $2F
		x"00", x"3E", x"51", x"49", x"45", x"3E", 	-- $30
		x"00", x"00", x"42", x"7F", x"40", x"00", 	-- $31
		x"00", x"62", x"51", x"49", x"49", x"46", 	-- $32
		x"00", x"22", x"49", x"49", x"49", x"36", 	-- $33
		x"00", x"18", x"14", x"12", x"7F", x"10", 	-- $34
		x"00", x"2F", x"49", x"49", x"49", x"31", 	-- $35
		x"00", x"3C", x"4A", x"49", x"49", x"30", 	-- $36
		x"00", x"01", x"71", x"09", x"05", x"03", 	-- $37
		x"00", x"36", x"49", x"49", x"49", x"36", 	-- $38
		x"00", x"06", x"49", x"49", x"29", x"1E", 	-- $39
		x"00", x"00", x"6C", x"6C", x"00", x"00", 	-- $3A
		x"00", x"00", x"EC", x"6C", x"00", x"00", 	-- $3B
		x"00", x"08", x"14", x"22", x"41", x"00", 	-- $3C
		x"00", x"24", x"24", x"24", x"24", x"24", 	-- $3D
		x"00", x"00", x"41", x"22", x"14", x"08", 	-- $3E
		x"00", x"02", x"01", x"59", x"09", x"06", 	-- $3F
		x"00", x"3E", x"41", x"5D", x"55", x"1E", 	-- $40
		x"00", x"7E", x"11", x"11", x"11", x"7E", 	-- $41
		x"00", x"7F", x"49", x"49", x"49", x"36", 	-- $42
		x"00", x"3E", x"41", x"41", x"41", x"22", 	-- $43
		x"00", x"7F", x"41", x"41", x"41", x"3E", 	-- $44
		x"00", x"7F", x"49", x"49", x"49", x"41", 	-- $45
		x"00", x"7F", x"09", x"09", x"09", x"01", 	-- $46
		x"00", x"3E", x"41", x"49", x"49", x"7A", 	-- $47
		x"00", x"7F", x"08", x"08", x"08", x"7F", 	-- $48
		x"00", x"00", x"41", x"7F", x"41", x"00", 	-- $49
		x"00", x"30", x"40", x"40", x"40", x"3F", 	-- $4A
		x"00", x"7F", x"08", x"14", x"22", x"41", 	-- $4B
		x"00", x"7F", x"40", x"40", x"40", x"40", 	-- $4C
		x"00", x"7F", x"02", x"04", x"02", x"7F", 	-- $4D
		x"00", x"7F", x"02", x"04", x"08", x"7F", 	-- $4E
		x"00", x"3E", x"41", x"41", x"41", x"3E", 	-- $4F
		x"00", x"7F", x"09", x"09", x"09", x"06", 	-- $50
		x"00", x"3E", x"41", x"51", x"21", x"5E", 	-- $51
		x"00", x"7F", x"09", x"09", x"19", x"66", 	-- $52
		x"00", x"26", x"49", x"49", x"49", x"32", 	-- $53
		x"00", x"01", x"01", x"7F", x"01", x"01", 	-- $54
		x"00", x"3F", x"40", x"40", x"40", x"3F", 	-- $55
		x"00", x"1F", x"20", x"40", x"20", x"1F", 	-- $56
		x"00", x"3F", x"40", x"3C", x"40", x"3F", 	-- $57
		x"00", x"63", x"14", x"08", x"14", x"63", 	-- $58
		x"00", x"07", x"08", x"70", x"08", x"07", 	-- $59
		x"00", x"71", x"49", x"45", x"43", x"00", 	-- $5A
		x"00", x"00", x"7F", x"41", x"41", x"00", 	-- $5B
		x"00", x"02", x"04", x"08", x"10", x"20", 	-- $5C
		x"00", x"00", x"41", x"41", x"7F", x"00", 	-- $5D
		x"00", x"04", x"02", x"01", x"02", x"04", 	-- $5E
		x"80", x"80", x"80", x"80", x"80", x"80", 	-- $5F
		x"00", x"00", x"03", x"07", x"00", x"00", 	-- $60
		x"00", x"20", x"54", x"54", x"54", x"78", 	-- $61
		x"00", x"7F", x"44", x"44", x"44", x"38", 	-- $62
		x"00", x"38", x"44", x"44", x"44", x"28", 	-- $63
		x"00", x"38", x"44", x"44", x"44", x"7F", 	-- $64
		x"00", x"38", x"54", x"54", x"54", x"08", 	-- $65
		x"00", x"08", x"7E", x"09", x"09", x"00", 	-- $66
		x"00", x"18", x"A4", x"A4", x"A4", x"7C", 	-- $67
		x"00", x"7F", x"04", x"04", x"78", x"00", 	-- $68
		x"00", x"00", x"00", x"7D", x"40", x"00", 	-- $69
		x"00", x"40", x"80", x"84", x"7D", x"00", 	-- $6A
		x"00", x"7F", x"10", x"28", x"44", x"00", 	-- $6B
		x"00", x"00", x"00", x"7F", x"40", x"00", 	-- $6C
		x"00", x"7C", x"04", x"18", x"04", x"78", 	-- $6D
		x"00", x"7C", x"04", x"04", x"78", x"00", 	-- $6E
		x"00", x"38", x"44", x"44", x"44", x"38", 	-- $6F
		x"00", x"FC", x"44", x"44", x"44", x"38", 	-- $70
		x"00", x"38", x"44", x"44", x"44", x"FC", 	-- $71
		x"00", x"44", x"78", x"44", x"04", x"08", 	-- $72
		x"00", x"08", x"54", x"54", x"54", x"20", 	-- $73
		x"00", x"04", x"3E", x"44", x"24", x"00", 	-- $74
		x"00", x"3C", x"40", x"20", x"7C", x"00", 	-- $75
		x"00", x"1C", x"20", x"40", x"20", x"1C", 	-- $76
		x"00", x"3C", x"60", x"30", x"60", x"3C", 	-- $77
		x"00", x"6C", x"10", x"10", x"6C", x"00", 	-- $78
		x"00", x"9C", x"A0", x"60", x"3C", x"00", 	-- $79
		x"00", x"64", x"54", x"54", x"4C", x"00", 	-- $7A
		x"00", x"08", x"3E", x"41", x"41", x"00", 	-- $7B
		x"00", x"00", x"00", x"77", x"00", x"00", 	-- $7C
		x"00", x"00", x"41", x"41", x"3E", x"08", 	-- $7D
		x"00", x"02", x"01", x"02", x"01", x"00", 	-- $7E
		x"00", x"3C", x"26", x"23", x"26", x"3C"	 	-- $7F
	);

end constants;

package body constants is

	function is_std_clock (signal cpow : in freq_index) return std_logic is
	begin
		if cpow = cpow_std then return '1';
		else return '0';
		end if;
	end;

	function is_slow_clock (signal cpow : in freq_index) return std_logic is
	begin
		if cpow <= cpow_slow then return '1';
		else return '0';
		end if;
	end;

end constants;
