--------------------------------------------------------------------------------
--
-- Low-level control for EA DOGM132 display, 32*132 pixel
-- ST7565R display controller with SPI bus.
--
-- jm 2.9.2016
--
--------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

library unisim;
use unisim.vcomponents.all;

library work;
use work.constants.all;

ENTITY lcd_controller IS
	PORT(
		-- connections to client in FPGA
		clk			: IN STD_LOGIC;  				-- system clock
		reset_n		: IN STD_LOGIC;  				-- active low reinitializes lcd
		enable		: IN STD_LOGIC;  				-- latches data into lcd controller
		data			: IN STD_LOGIC_VECTOR(8 DOWNTO 0);  -- data and a0 signals. present until busy goes low again
		busy			: OUT STD_LOGIC := '1';  	-- lcd controller busy/idle feedback

		-- connections to physical LCD
		si				: OUT STD_LOGIC;  			-- data
		scl			: OUT	STD_LOGIC;				-- clock
		cs_n			: OUT STD_LOGIC;				-- chip select, active low
		a0				: OUT STD_LOGIC;				-- high = data, low = command
		res_n			: OUT STD_LOGIC				-- reset, active low
	); 
END lcd_controller;

ARCHITECTURE controller OF lcd_controller IS

	-- Initialization data.
  
	constant pwr_delay: natural := 50000*CLK_IO_RATE;		-- power-on delay 50 ms
	constant res_delay: natural := 1000*CLK_IO_RATE;		-- reset delay 1 ms

	constant init_length: natural := 14;			-- number of bytes in my_init
	type init_table is array (natural range 0 to init_length-1) of std_logic_vector (8 downto 0);
	constant my_init: init_table :=
	(
		-- bit 8 = A0
		-- bit 7..0 = command data
		-- Initialization sequence for "low power" (2.4 .. 3.3V single supply)
		-- according to DOGM132 datasheet issue 05.2012
		
		"001000000",	-- display start line 0
		"010100000",	-- ADC normal 
		"011001000",	-- common output mode reversed
		"010100110",	-- display normal (not inverse)
		"010100010",	-- set bias 1/9 (duty 1/33)
		"000101111",	-- booster, regulator, follower ON
		"011111000",	-- set internal booster
		"000000000",	-- ... to 3x/4x
		"000100011",	-- set voltage regulator
		"010000001",	-- set contrast
		"000011111",	-- ... to 31 (range 1..63)
		"010101100",	-- static indicator
		"000000000",	-- ... none
		"010101111"		-- display ON
	);
  
	-- state control
	type control is (powerup, reset, endreset, init, ready, send);
	signal state: control := powerup;									-- main state machine
  
	signal clk_counter: natural range 0 to pwr_delay := 0;		-- delay during init
	signal bit_counter: integer range -1 to 7 := 0;					-- sub-state during regular sending
	signal init_counter: natural range 0 to init_length:= 0;		-- index to initalization sequence table
	signal init_active: std_logic := '0';
	signal lcd_data: std_logic_vector (8 downto 0);					-- current data for send state
  
	signal notclk: std_logic;
  
BEGIN

	-- Prepare IO clock for output via regular IO pin
   -- Clock forwarding circuit using the double data-rate register
   -- Spartan-3E/3A/6
   -- Xilinx HDL Language Template, version 14.7

	notclk <= NOT clk;

   ODDR2_inst : ODDR2
   generic map(
      DDR_ALIGNMENT => "NONE", -- Sets output alignment to "NONE", "C0", "C1" 
      INIT => '0', -- Sets initial state of the Q output to '0' or '1'
      SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" set/reset
   port map (
      Q => scl, 		-- 1-bit output data
      C0 => clk, 		-- 1-bit clock input
      C1 => notclk, 	-- 1-bit clock input
      CE => '1',  	-- 1-bit clock enable input
      D0 => '0',   	-- 1-bit data input (associated with C0)
      D1 => '1',   	-- 1-bit data input (associated with C1)
      R => '0',    	-- 1-bit reset input
      S => '0'     	-- 1-bit set input
   );
	
	process (clk)
	begin
		if rising_edge(clk) then 
			-- setup of data and cs_n on falling edge of SPI clock
			case state is 
			
				when powerup =>
					--
					-- initialize state machine and outputs
					--
					clk_counter <= pwr_delay;		-- power-up delay 50 ms
					init_counter <= 0;				-- start of initialization table	
					res_n <= '0';						-- pull reset low
					cs_n <= '1';						-- not selected
					busy <= '1';						-- this controller is busy
					state <= reset;					-- move on to init state
					
				when reset =>							
					--
					-- hold reset low until power stable (50 ms)
					--
					if clk_counter = 0 then			-- 50 ms are over
						res_n <= '1';					-- release reset
						clk_counter <= res_delay;	-- then wait 1ms
						state <= endreset;
					else 
						clk_counter <= clk_counter-1;
					end if;
					
				when endreset =>						
					--
					-- wait for recovery from reset (1ms)
					--
					if clk_counter = 0 then
						state <= init;
						init_active <= '1';
						init_counter <= 0;
					else 
						clk_counter <= clk_counter-1;
					end if;
									
				when init =>
					--
					-- run through my_init table of initialization data.
					-- clk_counter times the individual nibbles (counts down)		
					--
					if (init_counter = init_length) then	-- end of table, initialization complete
						init_active <= '0';
						state <= ready;
					else												-- init table not completed yet
						lcd_data <= my_init (init_counter);	-- data to be sent
						init_counter <= init_counter +1;		-- prepare for next table entry (after delay)
						bit_counter <= 7;     					-- send bits 7..0
						state <= send;
					end if;
				
				when ready => 
					--
					-- ready to receive data from client
					--
					busy <= '0';
					if (enable = '1') then 		-- have data to send to lcd
						lcd_data <= data;			-- data to be sent
						busy <= '1';				-- this controller is busy
						bit_counter <= 7;     	-- send bits 7..0
						state <= send;
					end if;
						
				when send =>
					--
					-- transmitting 8 bits, high bit first
					--
					if bit_counter = -1 then	-- all bits sent
						cs_n <= '1';
						if init_active = '1' then state <= init;
						else 
							busy <= '0';
							state <= ready;
						end if;
					else
						cs_n <= '0';								-- select LCD
						a0 <= lcd_data (8);
						si <= lcd_data (bit_counter);
						bit_counter <= bit_counter -1;
					end if;
							
			end case;
			
			--
			-- async. reset 
			--
			if (reset_n = '0') then
				state <= powerup;
			end if;

		end if;
	end process;
end controller;    
 